"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HistogramCell = exports.SVGDisplayer = exports.Base64PNGDisplayer = exports.LinkCellRenderer = exports.getTextCellRenderer = exports.ToolTipAdapter = exports.FloatingTooltip = exports.formatter = exports.makeData = exports.bakedData = void 0;
const lodash_1 = __importDefault(require("lodash"));
const react_1 = __importDefault(require("react"));
const react_dom_1 = require("react-dom");
const recharts_1 = require("recharts");
const RechartTooltip_1 = require("../../vendor/RechartTooltip");
const RechartExtra_1 = require("../../vendor/RechartExtra");
function defaultFormatter(value) {
    return lodash_1.default.isArray(value) && (0, RechartExtra_1.isNumOrStr)(value[0]) && (0, RechartExtra_1.isNumOrStr)(value[1])
        ? value.join(' ~ ')
        : value;
}
exports.bakedData = [
    {
        name: 'Page A',
        population: 4000,
    },
    {
        name: 'Page B',
        population: 3000,
    },
    {
        name: 'Page C',
        population: 2000,
    },
    {
        name: 'Page D',
        population: 2780,
    },
    {
        name: 'Page E',
        population: 1890,
    },
];
const makeData = (histogram) => {
    const accum = [];
    for (let i = 0; i < histogram.length; i++) {
        accum.push({
            name: `${i + 1}/${histogram.length}`,
            population: histogram[i],
        });
    }
    //console.log('accum', accum)
    return accum;
};
exports.makeData = makeData;
const formatter = (value, name, props) => {
    if (props.payload.name === 'longtail') {
        return [value, name];
    }
    else {
        return [value, props.payload.name];
    }
};
exports.formatter = formatter;
function FloatingTooltip({ items, x, y }) {
    const offset = 30;
    const renderedItems = items.map((name, value) => {
        const [realName, realValue] = name;
        const formattedVal = realValue === 0 ? '<1' : realValue;
        return (react_1.default.createElement(react_1.default.Fragment, null,
            react_1.default.createElement("dt", null, realName),
            react_1.default.createElement("dd", null,
                formattedVal,
                "%")));
    });
    return (0, react_dom_1.createPortal)(react_1.default.createElement("div", { className: "floating-tooltip", style: { position: 'absolute', top: y + offset, left: x + offset } },
        react_1.default.createElement("dl", null, renderedItems)), document.body);
}
exports.FloatingTooltip = FloatingTooltip;
const ToolTipAdapter = (args) => {
    const { active, formatter, payload } = args;
    if (active && payload && payload.length) {
        const renderContent2 = () => {
            //const items = (itemSorter ? _.sortBy(payload, itemSorter) : payload).map((entry, i) => {
            const items = payload.map((entry, i) => {
                if (entry.type === 'none') {
                    return null;
                }
                const finalFormatter = entry.formatter || formatter || defaultFormatter;
                const { value, name } = entry;
                let finalValue = value;
                let finalName = name;
                if (finalFormatter && finalValue !== null && finalName !== null) {
                    const formatted = finalFormatter(value, name, entry, i, payload);
                    if (Array.isArray(formatted)) {
                        [finalValue, finalName] = formatted;
                    }
                    else {
                        finalValue = formatted;
                    }
                }
                return [finalName, finalValue];
            });
            return items;
        };
        return (react_1.default.createElement("div", { className: "custom-tooltip" },
            react_1.default.createElement(FloatingTooltip, { items: renderContent2(), x: args.box.x, y: args.box.y })));
    }
    return null;
};
exports.ToolTipAdapter = ToolTipAdapter;
const getTextCellRenderer = (formatter) => {
    const TextCellRenderer = (props) => {
        return react_1.default.createElement("span", null, formatter(props));
    };
    return TextCellRenderer;
};
exports.getTextCellRenderer = getTextCellRenderer;
const LinkCellRenderer = (props) => {
    return react_1.default.createElement("a", { href: props.value }, props.value);
};
exports.LinkCellRenderer = LinkCellRenderer;
const Base64PNGDisplayer = (props) => {
    const imgString = 'data:image/png;base64,' + props.value;
    return react_1.default.createElement("img", { src: imgString });
};
exports.Base64PNGDisplayer = Base64PNGDisplayer;
const SVGDisplayer = (props) => {
    const markup = { __html: props.value };
    return (react_1.default.createElement("div", { dangerouslySetInnerHTML: markup }));
};
exports.SVGDisplayer = SVGDisplayer;
const HistogramCell = (props) => {
    //debugger;
    if (props === undefined || props.value === undefined) {
        return react_1.default.createElement("span", null);
    }
    const histogram = props.value;
    //for key "index", the value is "histogram"
    // this causes ReChart to blow up, so we check to see if it's an array
    if (histogram === undefined || !lodash_1.default.isArray(histogram)) {
        return react_1.default.createElement("span", null);
    }
    const dumbClickHandler = (rechartsArgs, _unused_react) => {
        // I can't find the type for rechartsArgs
        // these are probably the keys we care about
        // activeTooltipIndex
        // activeLabel
        console.log('dumbClickHandler', rechartsArgs);
    };
    return (react_1.default.createElement("div", { className: "histogram-component" },
        react_1.default.createElement(recharts_1.BarChart, { width: 100, height: 24, barGap: 1, data: histogram, onClick: dumbClickHandler },
            react_1.default.createElement("defs", null,
                react_1.default.createElement("pattern", { id: "star", width: "10", height: "10", patternUnits: "userSpaceOnUse" },
                    react_1.default.createElement("polygon", { stroke: "pink", points: "0,0 2,5 0,10 5,8 10,10 8,5 10,0 5,2" })),
                react_1.default.createElement("pattern", { id: "stripe", width: "4", height: "4", patternUnits: "userSpaceOnUse", patternTransform: "rotate(45)" },
                    react_1.default.createElement("rect", { width: "2", height: "4", fill: "red" })),
                react_1.default.createElement("pattern", { id: "circles", width: "4", height: "4", patternUnits: "userSpaceOnUse" },
                    react_1.default.createElement("circle", { "data-color": "outline", stroke: "pink", cx: ".5", cy: ".5", r: "1.5" })),
                react_1.default.createElement("pattern", { id: "checkers", x: "0", y: "0", width: "4", height: "4", patternUnits: "userSpaceOnUse" },
                    react_1.default.createElement("rect", { stroke: "#0f0", x: "0", width: "2", height: "2", y: "0" }),
                    react_1.default.createElement("rect", { x: "2", width: "2", height: "2", y: "2" })),
                react_1.default.createElement("pattern", { id: "leafs", x: "0", y: "0", width: "6", height: "6", patternUnits: "userSpaceOnUse", patternTransform: "translate(1, 1) rotate(0) skewX(0)" },
                    react_1.default.createElement("svg", { width: "5", height: "5", viewBox: "0 0 100 100" },
                        react_1.default.createElement("g", { fill: "teal", opacity: "1" },
                            react_1.default.createElement("path", { d: "M99.9557 99.9557C45.4895 98.3748 1.6248 54.5101 0.0439453 0.0439453C54.5101 1.6248 98.3748 45.4895 99.9557 99.9557Z" }))))),
            react_1.default.createElement(recharts_1.Bar, { dataKey: "population", stroke: "#000", fill: "gray", stackId: "stack" }),
            react_1.default.createElement(recharts_1.Bar, { dataKey: "tail", stroke: "#000", fill: "gray", stackId: "stack" }),
            react_1.default.createElement(recharts_1.Bar, { dataKey: "true", stroke: "#00f", fill: "#00f", stackId: "stack" }),
            react_1.default.createElement(recharts_1.Bar, { dataKey: "false", stroke: "#000", fill: "#fff", stackId: "stack" }),
            react_1.default.createElement(recharts_1.Bar, { dataKey: "cat_pop", stroke: "pink", fill: "url(#circles)", stackId: "stack" }),
            react_1.default.createElement(recharts_1.Bar, { dataKey: "unique", stroke: "#0f0", fill: "url(#checkers)", stackId: "stack" }),
            react_1.default.createElement(recharts_1.Bar, { dataKey: "longtail", stroke: "teal", fill: "url(#leafs)", stackId: "stack" }),
            react_1.default.createElement(recharts_1.Bar, { dataKey: "NA", fill: "url(#stripe)", stackId: "stack" }),
            react_1.default.createElement(RechartTooltip_1.Tooltip, { formatter: exports.formatter, labelStyle: { display: 'None' }, wrapperStyle: { zIndex: 999991 }, contentStyle: { color: 'black' }, content: react_1.default.createElement(exports.ToolTipAdapter, null), offset: 20, allowEscapeViewBox: { x: true } }))));
};
exports.HistogramCell = HistogramCell;
//# sourceMappingURL=HistogramCell.js.map