"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.extractSDFT = exports.getCellRendererSelector = exports.dfToAgrid = exports.extractSingleSeriesSummary = exports.getTooltip = exports.extractPinnedRows = exports.getStyler = exports.colorNotNull = exports.colorMap = exports.getHistoIndex = exports.addToColDef = void 0;
const colorMap_1 = require("../../baked_data/colorMap");
const DFWhole_1 = require("./DFWhole");
const lodash_1 = __importStar(require("lodash"));
const HistogramCell_1 = require("./HistogramCell");
const SeriesSummaryTooltip_1 = require("./SeriesSummaryTooltip");
const Displayer_1 = require("./Displayer");
// for now colDef stuff with less than 3 implementantions should stay in this file
// as implementations grow large or with many implmentations, they should move to separate files
// like displayer
function addToColDef(dispArgs, summary_stats_column) {
    const formatter = (0, Displayer_1.getFormatterFromArgs)(dispArgs);
    if (formatter !== undefined) {
        const colDefExtras = { valueFormatter: formatter };
        return colDefExtras;
    }
    if (lodash_1.default.includes(DFWhole_1.cellRendererDisplayers, dispArgs.displayer)) {
        const crArgs = dispArgs;
        return {
            cellRenderer: (0, Displayer_1.getCellRenderer)(crArgs),
        };
    }
    return undefined;
}
exports.addToColDef = addToColDef;
function getHistoIndex(val, histogram_edges) {
    /*
  np.histogram([1, 2, 3, 4,  10, 20, 30, 40, 300, 300, 400, 500], bins=5)
  ( [  8,       0,     2,     1,     1],
  [  1. , 100.8, 200.6, 300.4, 400.2, 500. ])
  The bottom matters for us, those are the endge
  
  this means that 8 values are between 1 and 100.8  and 2 values are between 200.6 and 300.4
    */
    if (histogram_edges.length === 0) {
        return 0;
    }
    for (let i = 0; i < histogram_edges.length; i++) {
        if (val <= histogram_edges[i]) {
            return i;
        }
    }
    return histogram_edges.length;
}
exports.getHistoIndex = getHistoIndex;
function colorMap(cmr, histogram_edges) {
    // https://colorcet.com/gallery.html#isoluminant
    // https://github.com/holoviz/colorcet/tree/main/assets/CET
    // https://github.com/bokeh/bokeh/blob/ed285b11ab196e72336b47bf12f44e1bef5abed3/src/bokeh/models/mappers.py#L304
    const maps = {
        BLUE_TO_YELLOW: colorMap_1.BLUE_TO_YELLOW,
        DIVERGING_RED_WHITE_BLUE: colorMap_1.DIVERGING_RED_WHITE_BLUE,
    };
    const cmap = maps[cmr.map_name];
    function numberToColor(val) {
        const histoIndex = getHistoIndex(val, histogram_edges);
        const scaledIndex = Math.round((histoIndex / histogram_edges.length) * cmap.length);
        return cmap[scaledIndex];
    }
    function cellStyle(params) {
        const val = cmr.val_column ? params.data[cmr.val_column] : params.value;
        const color = numberToColor(val);
        return {
            backgroundColor: color,
        };
    }
    const retProps = {
        cellStyle: cellStyle,
    };
    return retProps;
}
exports.colorMap = colorMap;
function colorNotNull(cmr) {
    function cellStyle(params) {
        if (params.data === undefined) {
            return { backgroundColor: 'inherit' };
        }
        const val = params.data[cmr.exist_column];
        const valPresent = val && val !== null;
        const isPinned = params.node.rowPinned;
        const color = valPresent && !isPinned ? cmr.conditional_color : 'inherit';
        return {
            backgroundColor: color,
        };
    }
    const retProps = {
        cellStyle: cellStyle,
    };
    return retProps;
}
exports.colorNotNull = colorNotNull;
function getStyler(cmr, col_name, histogram_stats) {
    switch (cmr.color_rule) {
        case 'color_map': {
            //block necessary because you cant define varaibles in case blocks
            const statsCol = cmr.val_column || col_name;
            const summary_stats_cell = histogram_stats[statsCol];
            if (summary_stats_cell &&
                summary_stats_cell.histogram_bins !== undefined) {
                return colorMap(cmr, summary_stats_cell.histogram_bins);
            }
            else {
                console.log('histogram bins not found for color_map');
                return {};
            }
        }
        case 'color_not_null':
            return colorNotNull(cmr);
    }
}
exports.getStyler = getStyler;
function extractPinnedRows(sdf, prc) {
    return lodash_1.default.map(lodash_1.default.map(prc, 'primary_key_val'), (x) => lodash_1.default.find(sdf, { index: x }));
}
exports.extractPinnedRows = extractPinnedRows;
function getTooltip(ttc, single_series_summary_df) {
    switch (ttc.tooltip_type) {
        case 'simple':
            return { tooltipField: ttc.val_column };
        case 'summary_series':
            return {
                tooltipComponent: (0, SeriesSummaryTooltip_1.getBakedDFViewer)(single_series_summary_df),
                tooltipField: 'index',
                tooltipComponentParams: {},
            };
    }
}
exports.getTooltip = getTooltip;
function extractSingleSeriesSummary(full_summary_stats_df, col_name) {
    return {
        dfviewer_config: {
            column_config: [
                { col_name: 'index', displayer_args: { displayer: 'obj' } },
                { col_name: col_name, displayer_args: { displayer: 'obj' } },
            ],
            pinned_rows: [],
        },
        data: lodash_1.default.filter(lodash_1.default.map(full_summary_stats_df, (row) => lodash_1.default.pick(row, ['index', col_name])), { index: 'dtype' }),
    };
}
exports.extractSingleSeriesSummary = extractSingleSeriesSummary;
function dfToAgrid(tdf, dfviewer_config, full_summary_stats_df) {
    //more convienient df format for some formatters
    const hdf = extractSDFT(full_summary_stats_df || []);
    const retColumns = dfviewer_config.column_config.map((f) => {
        const single_series_summary_df = extractSingleSeriesSummary(full_summary_stats_df, f.col_name);
        const color_map_config = f.color_map_config
            ? getStyler(f.color_map_config, f.col_name, hdf)
            : {};
        const tooltip_config = f.tooltip_config
            ? getTooltip(f.tooltip_config, single_series_summary_df)
            : {};
        const colDef = Object.assign(Object.assign(Object.assign(Object.assign({ field: f.col_name, headerName: f.col_name, cellDataType: false, cellStyle: {} }, addToColDef(f.displayer_args, hdf[f.col_name])), color_map_config), tooltip_config), f.ag_grid_specs);
        return colDef;
    });
    return [retColumns, tdf];
}
exports.dfToAgrid = dfToAgrid;
// this is very similar to the colDef parts of dfToAgrid
function getCellRendererSelector(pinned_rows) {
    const anyRenderer = {
        component: (0, HistogramCell_1.getTextCellRenderer)(Displayer_1.objFormatter),
    };
    return (params) => {
        var _a;
        if (params.node.rowPinned) {
            const pk = lodash_1.default.get(params.node.data, 'index');
            if (pk === undefined) {
                return anyRenderer; // default renderer
            }
            const maybePrc = lodash_1.default.find(pinned_rows, {
                primary_key_val: pk,
            });
            if (maybePrc === undefined) {
                return anyRenderer;
            }
            const prc = maybePrc;
            console.log('params', params);
            const currentCol = (_a = params.column) === null || _a === void 0 ? void 0 : _a.getColId();
            if ((prc.default_renderer_columns === undefined &&
                currentCol === 'index') ||
                lodash_1.default.includes(prc.default_renderer_columns, currentCol)) {
                return anyRenderer;
            }
            const possibCellRenderer = (0, Displayer_1.getCellRenderer)(prc.displayer_args);
            if (possibCellRenderer === undefined) {
                const formattedRenderer = {
                    component: (0, HistogramCell_1.getTextCellRenderer)((0, Displayer_1.getFormatter)(prc.displayer_args)),
                };
                return formattedRenderer;
            }
            return { component: possibCellRenderer };
        }
        else {
            return undefined; // rows that are not pinned don't use a row level cell renderer
        }
    };
}
exports.getCellRendererSelector = getCellRendererSelector;
function extractSDFT(summaryStatsDf) {
    const maybeHistogramBins = lodash_1.default.find(summaryStatsDf, { index: 'histogram_bins' }) || {};
    const maybeHistogramLogBins = lodash_1.default.find(summaryStatsDf, { index: 'histogram_log_bins' }) || {};
    const allColumns = lodash_1.default.without(lodash_1.default.union(lodash_1.default.keys(maybeHistogramBins), lodash_1.default.keys(maybeHistogramLogBins)), 'index');
    const vals = lodash_1.default.map(allColumns, (colName) => {
        return {
            histogram_bins: lodash_1.default.get(maybeHistogramBins, colName, []),
            histogram_log_bins: lodash_1.default.get(maybeHistogramLogBins, colName, []),
        };
    });
    return (0, lodash_1.zipObject)(allColumns, vals);
}
exports.extractSDFT = extractSDFT;
/*
I would love for extractSDF to be more elegant like the following function.  I just can't quite get it to work
time to move on

export function extractSDFT2(summaryStatsDf:DFData) : SDFT  {
  const rows = ['histogram_bins', 'histogram_log_bins']

  const extracted = _.map(rows, (pk) => {
    return _.find(summaryStatsDf,  {'index': pk}) || {}
  })
  const dupKeys: string[][] = _.map(extracted, _.keys);
  const allColumns: string[] = _.without(_.union(...dupKeys), 'index');
  const vals:SDFMeasure[] = _.map(allColumns, (colName) => {
    const pairs = _.map(_.zip(rows, extracted), (rname, row) => {
      return [rname, (_.get(row, colName, []) as number[])];
    })
    return _.fromPairs(pairs) as SDFMeasure;
  });
  return zipObject(allColumns, vals) as SDFT;
}
*/
//# sourceMappingURL=gridUtils.js.map