"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ArgGetters = exports.OperationDetail = void 0;
const lodash_1 = __importDefault(require("lodash"));
const utils_1 = require("./utils");
const react_1 = __importDefault(require("react"));
const OperationDetail = ({ command, setCommand, columns, commandPatterns, }) => {
    if (command === undefined) {
        return react_1.default.createElement("span", null);
    }
    const commandName = command[0]['symbol'];
    const pattern = commandPatterns[commandName];
    if (!lodash_1.default.isArray(pattern)) {
        //we shouldn't get here
        return react_1.default.createElement("h2", null,
            "unknown command ",
            commandName);
    }
    else if (lodash_1.default.isEqual(pattern, [null])) {
        return react_1.default.createElement("div", { className: "operation-detail" });
    }
    else {
        const fullPattern = pattern;
        return (react_1.default.createElement("div", { className: "operation-detail" },
            react_1.default.createElement(exports.ArgGetters, { command: command, fullPattern: fullPattern, setCommand: setCommand, columns: columns })));
    }
    return react_1.default.createElement("h2", null);
};
exports.OperationDetail = OperationDetail;
const ArgGetters = ({ command, fullPattern, setCommand, columns, }) => {
    /* reads the argspec and sets up the proper getters/setters */
    const makeArgGetter = (pattern) => {
        const idx = pattern[0];
        const val = command[idx];
        const valSetter = (newVal) => {
            const newCommand = (0, utils_1.replaceAtIdx)(command, idx, newVal);
            //console.log('newCommand', newCommand);
            setCommand(newCommand);
        };
        return (react_1.default.createElement("div", { key: idx },
            react_1.default.createElement(ArgGetter, { argProps: pattern, renderKey: idx, val: val, setter: valSetter, columns: columns })));
    };
    return react_1.default.createElement("div", { className: "arg-getters" }, fullPattern.map(makeArgGetter));
};
exports.ArgGetters = ArgGetters;
const ArgGetter = ({ argProps, val, setter, columns, renderKey, }) => {
    const [_argPos, label, argType, lastArg] = argProps;
    const defaultShim = (event) => setter(event.target.value);
    if (argType === 'enum' && lodash_1.default.isArray(lastArg)) {
        return (react_1.default.createElement("fieldset", { key: renderKey },
            react_1.default.createElement("label", null,
                " ",
                label,
                " "),
            react_1.default.createElement("select", { defaultValue: val, onChange: defaultShim }, lastArg.map((optionVal) => (react_1.default.createElement("option", { key: optionVal, value: optionVal }, optionVal))))));
    }
    else if (argType === 'type') {
        if (lastArg === 'integer') {
            const valSetterShim = (event) => setter(parseInt(event.target.value));
            return (react_1.default.createElement("fieldset", { key: renderKey },
                react_1.default.createElement("label", null,
                    " ",
                    label,
                    " "),
                react_1.default.createElement("input", { type: "number", defaultValue: val, step: "1", onChange: valSetterShim })));
        }
        else if (lastArg === 'float') {
            const valSetterShim = (event) => setter(parseFloat(event.target.value));
            return (react_1.default.createElement("fieldset", { key: renderKey },
                react_1.default.createElement("label", null,
                    " ",
                    label,
                    " "),
                react_1.default.createElement("input", { type: "number", step: "0.01", defaultValue: val, onChange: valSetterShim })));
        }
        else if (lastArg === 'string') {
            const valSetterShim = (event) => setter(event.target.value);
            return (react_1.default.createElement("fieldset", { key: renderKey },
                react_1.default.createElement("label", null,
                    " ",
                    label,
                    " "),
                react_1.default.createElement("input", { type: "text", defaultValue: val, onChange: valSetterShim })));
        }
        else {
            return (react_1.default.createElement("fieldset", { key: renderKey },
                react_1.default.createElement("label", null,
                    " ",
                    label,
                    " "),
                react_1.default.createElement("input", { value: "dont know" })));
        }
    }
    else if (argType === 'colEnum') {
        const widgetRow = columns.map((colName) => {
            const colSetter = (event) => {
                const newColVal = event.target.value;
                if (lodash_1.default.isString(newColVal)) {
                    const updatedColDict = (0, utils_1.replaceAtKey)(val, colName, newColVal); // as Record<string, string>
                    setter((0, utils_1.objWithoutNull)(updatedColDict, ['null']));
                }
            };
            const colVal = lodash_1.default.get(val, colName, 'null');
            if (!lodash_1.default.isArray(lastArg)) {
                return react_1.default.createElement("h3", null, " arg error");
            }
            return (react_1.default.createElement("td", { key: renderKey + colName },
                react_1.default.createElement("select", { defaultValue: colVal, onChange: colSetter }, lastArg.map((optionVal) => (react_1.default.createElement("option", { key: optionVal, value: optionVal }, optionVal))))));
        });
        return (react_1.default.createElement("div", { className: "col-enum", key: renderKey },
            react_1.default.createElement("table", null,
                react_1.default.createElement("thead", null,
                    react_1.default.createElement("tr", null, columns.map((colName) => (react_1.default.createElement("th", { key: colName }, colName))))),
                react_1.default.createElement("tbody", null,
                    react_1.default.createElement("tr", null, widgetRow)))));
    }
    else {
        return react_1.default.createElement("h3", null, " unknown argtype ");
    }
};
//# sourceMappingURL=OperationDetail.js.map