/**
 * Reimplementation of rechart tooltips.  most of this is copied from ??? in the recharts source tree because the parts we are editting are deeply embedded into functions.

 Most of the point of this is to get tooltips to be absolutely positioned off of document.body, not the nearest parent element.  AG-Grid hides those tooltips otherwise.
 */
import React, { CSSProperties, ReactNode, ReactElement, SVGProps } from 'react';
import type { ValueType, NameType, Payload, DefaultProps } from './RechartExtra';
import { AnimationDuration, AnimationTiming } from './RechartExtra';
export type ContentType<TValue extends ValueType, TName extends NameType> = ReactElement | ((props: TooltipProps<TValue, TName>) => ReactNode);
type UniqueFunc<TValue extends ValueType, TName extends NameType> = (entry: Payload<TValue, TName>) => unknown;
type UniqueOption<TValue extends ValueType, TName extends NameType> = boolean | UniqueFunc<TValue, TName>;
export type OptionalCoords = {
    x?: number;
    y?: number;
};
export type TooltipProps<TValue extends ValueType, TName extends NameType> = DefaultProps<TValue, TName> & {
    allowEscapeViewBox?: {
        x?: boolean;
        y?: boolean;
    };
    reverseDirection?: {
        x?: boolean;
        y?: boolean;
    };
    content?: ContentType<TValue, TName>;
    viewBox?: {
        x?: number;
        y?: number;
        width?: number;
        height?: number;
    };
    active?: boolean;
    offset?: number;
    wrapperStyle?: CSSProperties;
    cursor?: boolean | ReactElement | SVGProps<SVGElement>;
    coordinate?: OptionalCoords;
    position?: OptionalCoords;
    trigger?: 'hover' | 'click';
    shared?: boolean;
    payloadUniqBy?: UniqueOption<TValue, TName>;
    isAnimationActive?: boolean;
    animationDuration?: AnimationDuration;
    animationEasing?: AnimationTiming;
    filterNull?: boolean;
    useTranslate3d?: boolean;
    box?: OptionalCoords;
};
export declare const Tooltip: {
    <TValue extends ValueType, TName extends NameType>(props: DefaultProps<TValue, TName> & {
        allowEscapeViewBox?: {
            x?: boolean | undefined;
            y?: boolean | undefined;
        } | undefined;
        reverseDirection?: {
            x?: boolean | undefined;
            y?: boolean | undefined;
        } | undefined;
        content?: ContentType<TValue, TName> | undefined;
        viewBox?: {
            x?: number | undefined;
            y?: number | undefined;
            width?: number | undefined;
            height?: number | undefined;
        } | undefined;
        active?: boolean | undefined;
        offset?: number | undefined;
        wrapperStyle?: React.CSSProperties | undefined;
        cursor?: boolean | React.ReactElement<any, string | React.JSXElementConstructor<any>> | React.SVGProps<SVGElement> | undefined;
        coordinate?: OptionalCoords | undefined;
        position?: OptionalCoords | undefined;
        trigger?: "hover" | "click" | undefined;
        shared?: boolean | undefined;
        payloadUniqBy?: UniqueOption<TValue, TName> | undefined;
        isAnimationActive?: boolean | undefined;
        animationDuration?: number | undefined;
        animationEasing?: AnimationTiming | undefined;
        filterNull?: boolean | undefined;
        useTranslate3d?: boolean | undefined;
        box?: OptionalCoords | undefined;
    } & {
        children?: React.ReactNode;
    }): React.JSX.Element;
    displayName: string;
    /**
     * needs to be set so that renderByOrder can access an have default values for
     * children.props when there are no props set by the consumer
     * doesn't work if using default parameters
     */
    defaultProps: TooltipProps<number, string>;
};
export {};
