"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Tooltip = void 0;
//@ts-nocheck
/**
 * Reimplementation of rechart tooltips.  most of this is copied from ??? in the recharts source tree because the parts we are editting are deeply embedded into functions.

 Most of the point of this is to get tooltips to be absolutely positioned off of document.body, not the nearest parent element.  AG-Grid hides those tooltips otherwise.
 */
const react_1 = __importStar(require("react"));
const react_smooth_1 = require("react-smooth");
const lodash_1 = __importDefault(require("lodash"));
//import { Global } from '../util/Global';
//import { isNumber } from '../util/DataUtils';
const RechartExtra_1 = require("./RechartExtra");
const recharts_1 = require("recharts");
const CLS_PREFIX = 'recharts-tooltip-wrapper';
const EPS = 1;
function defaultUniqBy(entry) {
    return entry.dataKey;
}
function getUniqPayload(option, payload) {
    if (option === true) {
        //@ts-ignore
        return lodash_1.default.uniqBy(payload, defaultUniqBy);
    }
    if (lodash_1.default.isFunction(option)) {
        return lodash_1.default.uniqBy(payload, option);
    }
    return payload;
}
function renderContent(content, props) {
    if (react_1.default.isValidElement(content)) {
        return react_1.default.cloneElement(content, props);
    }
    if (lodash_1.default.isFunction(content)) {
        return react_1.default.createElement(content, props);
    }
    return react_1.default.createElement(recharts_1.DefaultTooltipContent, Object.assign({}, props));
}
const tooltipDefaultProps = {
    active: false,
    allowEscapeViewBox: { x: false, y: false },
    reverseDirection: { x: false, y: false },
    offset: 10,
    viewBox: { x: 0, y: 0, height: 0, width: 0 },
    coordinate: { x: 0, y: 0 },
    // this doesn't exist on TooltipProps
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    // @ts-ignore
    cursorStyle: {},
    separator: ' : ',
    wrapperStyle: {},
    contentStyle: {},
    itemStyle: {},
    labelStyle: {},
    cursor: true,
    trigger: 'hover',
    isAnimationActive: !RechartExtra_1.Global.isSsr,
    animationEasing: 'ease',
    animationDuration: 400,
    filterNull: true,
    useTranslate3d: false,
    box: { x: 0, y: 0 },
};
const Tooltip = (props) => {
    const [boxWidth, setBoxWidth] = (0, react_1.useState)(-1);
    const [boxHeight, setBoxHeight] = (0, react_1.useState)(-1);
    const [dismissed, setDismissed] = (0, react_1.useState)(false);
    const [dismissedAtCoordinate, setDismissedAtCoordinate] = (0, react_1.useState)({
        x: 0,
        y: 0,
    });
    const [boxCoords, setBoxCoords] = (0, react_1.useState)({ x: 0, y: 0 });
    const wrapperNode = (0, react_1.useRef)();
    const { allowEscapeViewBox, reverseDirection, coordinate, offset, position, viewBox, } = props;
    const handleKeyDown = (0, react_1.useCallback)((event) => {
        if (event.key === 'Escape') {
            setDismissed(true);
            setDismissedAtCoordinate((prev) => (Object.assign(Object.assign({}, prev), { x: coordinate === null || coordinate === void 0 ? void 0 : coordinate.x, y: coordinate === null || coordinate === void 0 ? void 0 : coordinate.y })));
        }
    }, [coordinate === null || coordinate === void 0 ? void 0 : coordinate.x, coordinate === null || coordinate === void 0 ? void 0 : coordinate.y]);
    (0, react_1.useEffect)(() => {
        const updateBBox = () => {
            if (dismissed) {
                document.removeEventListener('keydown', handleKeyDown);
                if ((coordinate === null || coordinate === void 0 ? void 0 : coordinate.x) !== dismissedAtCoordinate.x ||
                    (coordinate === null || coordinate === void 0 ? void 0 : coordinate.y) !== dismissedAtCoordinate.y) {
                    setDismissed(false);
                }
            }
            else {
                document.addEventListener('keydown', handleKeyDown);
            }
            if (wrapperNode.current && wrapperNode.current.getBoundingClientRect) {
                const box = wrapperNode.current.getBoundingClientRect();
                setBoxCoords({ x: box.x, y: box.y });
                if (Math.abs(box.width - boxWidth) > EPS ||
                    Math.abs(box.height - boxHeight) > EPS) {
                    setBoxWidth(box.width);
                    setBoxHeight(box.height);
                }
            }
            else if (boxWidth !== -1 || boxHeight !== -1) {
                setBoxWidth(-1);
                setBoxHeight(-1);
            }
        };
        updateBBox();
        return () => {
            document.removeEventListener('keydown', handleKeyDown);
        };
    }, [
        boxHeight,
        boxWidth,
        coordinate,
        dismissed,
        dismissedAtCoordinate.x,
        dismissedAtCoordinate.y,
        handleKeyDown,
    ]);
    const getTranslate = ({ key, tooltipDimension, viewBoxDimension, }) => {
        if (position && (0, RechartExtra_1.isNumber)(position[key])) {
            return position[key];
        }
        const pCoordinate = coordinate || { x: 0, y: 0 };
        const negative = (pCoordinate[key] || 0) - tooltipDimension - (offset || 0);
        const positive = (pCoordinate[key] || 0) + offset;
        if (allowEscapeViewBox === null || allowEscapeViewBox === void 0 ? void 0 : allowEscapeViewBox[key]) {
            return reverseDirection[key] ? negative : positive;
        }
        if (reverseDirection === null || reverseDirection === void 0 ? void 0 : reverseDirection[key]) {
            const tooltipBoundary = negative;
            const viewBoxBoundary = viewBox[key];
            if (tooltipBoundary < viewBoxBoundary) {
                return Math.max(positive, viewBox[key]);
            }
            return Math.max(negative, viewBox[key]);
        }
        const tooltipBoundary = positive + tooltipDimension;
        const viewBoxBoundary = viewBox[key] + viewBoxDimension;
        if (tooltipBoundary > viewBoxBoundary) {
            return Math.max(negative, viewBox[key]);
        }
        return Math.max(positive, viewBox[key]);
    };
    const { payload, payloadUniqBy, filterNull, active, wrapperStyle, useTranslate3d, isAnimationActive, animationDuration, animationEasing, } = props;
    const finalPayload = getUniqPayload(payloadUniqBy, filterNull && payload && payload.length
        ? payload.filter((entry) => !lodash_1.default.isNil(entry.value))
        : payload);
    const hasPayload = finalPayload && finalPayload.length;
    const { content } = props;
    let outerStyle = Object.assign({ pointerEvents: 'none', visibility: !dismissed && active && hasPayload ? 'visible' : 'hidden', position: 'absolute', top: 0, left: 0 }, wrapperStyle);
    let translateX, translateY;
    if (position && (0, RechartExtra_1.isNumber)(position.x) && (0, RechartExtra_1.isNumber)(position.y)) {
        translateX = position.x;
        translateY = position.y;
    }
    else if (boxWidth > 0 && boxHeight > 0 && coordinate) {
        translateX = getTranslate({
            key: 'x',
            tooltipDimension: boxWidth,
            viewBoxDimension: viewBox.width,
        });
        translateY = getTranslate({
            key: 'y',
            tooltipDimension: boxHeight,
            viewBoxDimension: viewBox.height,
        });
    }
    else {
        outerStyle.visibility = 'hidden';
    }
    outerStyle = Object.assign(Object.assign({}, (0, react_smooth_1.translateStyle)({
        transform: useTranslate3d
            ? `translate3d(${translateX}px, ${translateY}px, 0)`
            : `translate(${translateX}px, ${translateY}px)`,
    })), outerStyle);
    if (isAnimationActive && active) {
        outerStyle = Object.assign(Object.assign({}, (0, react_smooth_1.translateStyle)({
            transition: `transform ${animationDuration}ms ${animationEasing}`,
        })), outerStyle);
    }
    // eslint-disable-next-line
    // const cls = classNames(CLS_PREFIX, {
    //   [`${CLS_PREFIX}-right`]: isNumber(translateX) && coordinate && isNumber(coordinate.x) && translateX >= coordinate.x,
    //   [`${CLS_PREFIX}-left`]: isNumber(translateX) && coordinate && isNumber(coordinate.x) && translateX < coordinate.x,
    //   [`${CLS_PREFIX}-bottom`]:
    //     isNumber(translateY) && coordinate && isNumber(coordinate.y) && translateY >= coordinate.y,
    //   [`${CLS_PREFIX}-top`]: isNumber(translateY) && coordinate && isNumber(coordinate.y) && translateY < coordinate.y,
    // });
    const cls = `${CLS_PREFIX}-top`;
    return (
    // ESLint is disabled to allow listening to the `Escape` key. Refer to
    // https://github.com/recharts/recharts/pull/2925
    // eslint-disable-next-line jsx-a11y/no-noninteractive-element-interactions
    react_1.default.createElement("div", { tabIndex: -1, role: "dialog", className: cls, style: outerStyle, ref: wrapperNode }, renderContent(content, Object.assign(Object.assign({}, props), { box: boxCoords, payload: finalPayload }))));
};
exports.Tooltip = Tooltip;
// needs to be set so that renderByOrder can find the correct handler function
exports.Tooltip.displayName = 'Tooltip';
/**
 * needs to be set so that renderByOrder can access an have default values for
 * children.props when there are no props set by the consumer
 * doesn't work if using default parameters
 */
exports.Tooltip.defaultProps = tooltipDefaultProps;
//# sourceMappingURL=RechartTooltip.js.map